# $Id: GlossaryService.py 19160 2006-03-10 21:40:23Z schluep $
# Copyright (c) 2005-2006, Samuel Schluep, ETH Zurich. All rights reserved.
# See also LICENSE.txt

# Zope
from OFS.SimpleItem import SimpleItem
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
# Silva
from Products.Silva.transform.rendererreg import getRendererRegistry
from Products.Silva import SilvaPermissions
from Products.Silva.helpers import add_and_edit
# other products
from Products.Formulator.Form import ZMIForm
from Products.Formulator.Errors import FormValidationError
from Products.Formulator import StandardFields


_DEFAULT_LINK_RENDERER = 'globals_silva_glossary/link_renderer'
_DEFAULT_CONTENT_RENDERER = 'globals_silva_glossary/content_renderer'
_DEFAULT_GLOSSARY_RENDERER = 'globals_silva_glossary/glossary_renderer'


class GlossaryService(SimpleItem):
    """Service for the Silva Glossary product

    This service enables customization of the rendering for the:

    - the glossary links, i.e. the style of the text in a document which
      points to a glossary entry and the interactive functionality (e.g.
      onmouseover attributes),
    - the glossary content, i.e. the markup, style and functionality of
      the glossary entries embedded in to the document's view (e.g.
      rendering the glossary entries as tooltips),
    - the glossary page, i.e. the layout of the glossary page.

    These three renderers are defined by Zope Page Templates or Python
    Scripts and are referred by a path that is looked up using Zope's
    acquisition mechanism in the context of the document or glossary page
    to be rendered. Initially, default renderers are provided that may be
    used as examples for customization. The renderers can be reset to
    their default values leaving the field blank.
    """
    security = ClassSecurityInfo()

    meta_type = 'Silva Glossary Service'
    manage_options = (
        {'label': 'Edit', 'action': 'manage_glossaryServiceEditForm'},
        ) + SimpleItem.manage_options

    security.declareProtected(
        'View management screens', 'manage_glossaryServiceForm')
    manage_glossaryServiceEditForm = PageTemplateFile(
        'www/glossaryServiceEdit', globals(),
        __name__='manage_glossaryServiceEditForm')

    security.declareProtected('View management screens', 'manage_main')
    manage_main = manage_glossaryServiceEditForm # used by add_and_edit()

    def __init__(self, id, title):
        self.id = id
        self.title = title
        self._link_renderer_path = _DEFAULT_LINK_RENDERER
        self._content_renderer_path = _DEFAULT_CONTENT_RENDERER
        self._glossary_renderer_path = _DEFAULT_GLOSSARY_RENDERER

    # MANIPULATORS
    security.declareProtected(SilvaPermissions.ViewManagementScreens,
                              'manage_glossaryServiceEdit')
    def manage_glossaryServiceEdit(self, link_renderer_path='',
            content_renderer_path='', glossary_renderer_path='',
            REQUEST=None):
        """Manage method to set renderers.

        Specifing an empty string will reset the renderers to the
        Silva Glossary default renderers.

        link_renderer_path -- relative URL of the link renderer
        content_renderer_path -- relative URL of the content renderer
        glossary_renderer_path -- relative URL of the glossary page renderer
        """
        if link_renderer_path.lower().strip() == '':
            self._link_renderer_path = _DEFAULT_LINK_RENDERER
        else:
            self._link_renderer_path = link_renderer_path
        if content_renderer_path.lower().strip() == '':
            self._content_renderer_path = _DEFAULT_CONTENT_RENDERER
        else:
            self._content_renderer_path = content_renderer_path
        if glossary_renderer_path.lower().strip() == '':
            self._glossary_renderer_path = _DEFAULT_GLOSSARY_RENDERER
        else:
            self._glossary_renderer_path = glossary_renderer_path

        if REQUEST is not None:
            return self.manage_glossaryServiceEditForm(
                manage_tabs_message='Settings Changed')

    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'link_renderer_path')
    def link_renderer_path(self):
        """Gets the link renderer path

        Returns the relative URL of the link renderer.
        """
        return self._link_renderer_path

    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'content_renderer_path')
    def content_renderer_path(self):
        """Gets the content renderer path
        
        Returns the relative URL of the glossary entires content renderer.
        """
        return self._content_renderer_path

    security.declareProtected(
        SilvaPermissions.ChangeSilvaContent, 'glossary_renderer_path')
    def glossary_renderer_path(self):
        """Gets glossary page renderer path
        
        Returns the relative URL of the glossary page renderer.
        """
        try:
            return self._glossary_renderer_path
        except AttributeError:
            # XXX auto upgrade: add new attribute, may be removed later
            self._glossary_renderer_path = _DEFAULT_GLOSSARY_RENDERER
            return self._glossary_renderer_path

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'render_link')
    def render_link(self, gloss_id, gloss_entry, link_content):
        """Renders the glossary link using the specified link renderer.

        gloss_id -- XML id attribute to which the link should point to
        gloss_entry -- the Silva Glossary Entry object referred
        link_content -- the content (text) marked up by the link
        """
        # look up renderer
        link_renderer = self.REQUEST.model.unrestrictedTraverse(
            self._link_renderer_path)
        # render link and pass parameters
        rendered_link = link_renderer(gloss_id=gloss_id,
                             gloss_entry=gloss_entry,
                             link_content=link_content)
        if type(rendered_link) != 'unicode':
            rendered_link = unicode(rendered_link, 'utf-8')
        return rendered_link

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'render_content')
    def render_content(self):
        """Renders the glossary entry in the document (e.g. as tooltip)
        """
        # look up renderer
        content_renderer = self.REQUEST.model.unrestrictedTraverse(
            self._content_renderer_path)
        # render glossary entries (e.g. as tooltip)
        return content_renderer()

    security.declareProtected(
        SilvaPermissions.AccessContentsInformation, 'render_glossary')
    def render_glossary(self):
        """Renders all glossary entries in the Silva Glossary page

        The glossary renderer can access the list of glossary entries
        through something like "model.get_glossary_entries()"
        """
        # look up renderer
        glossary_renderer = self.REQUEST.model.unrestrictedTraverse(
            self._glossary_renderer_path)
        # render the list of glossary items
        return glossary_renderer()


InitializeClass(GlossaryService)

manage_addGlossaryServiceForm = PageTemplateFile(
    "www/glossaryServiceAdd", globals())

def manage_addGlossaryService(self, id, title="", REQUEST=None):
    """Add renderer registry service."""
    obj = GlossaryService(id, title)
    self._setObject(id, obj)
    ob = getattr(self, id)
    add_and_edit(self, id, REQUEST)
    return ''

