# $Id: glossarizer.py 306 2005-12-21 18:41:46Z eric $
# Copyright (c) 2005, Samuel Schluep, ETH Zurich. All rights reserved.
# See also LICENSE.txt

# Silva
from Products.Silva.adapters import adapter
from Products.Silva.interfaces import IGhost, IContainer
from interfaces import IGlossarizable
from Products.SilvaGlossary.helpers import glossaryLinkHelper

class GlossarizerAdapter(adapter.Adapter):
    """Default adapter class to search for glossary entries
    
    Does nothing, the method returns an empty dictionary.
    """
    __implements__ = (IGlossarizable)

    def getGlossaryEntries(self):
        """Returns an empty dictionary"""
        return {}


class DocumentGlossarizerAdapter(GlossarizerAdapter):
    """Adapter to search for glossary entries in Silva XML documents"""

    def getGlossaryEntries(self):
        """Get the glossary entries as a dictionary for Silva Document

        Can be used for Silva Document like content types.
        The key of the dictionary is the absolute path of the associated
        glossary entry. The value is the Glossary Entry object itself.
        """
        version = self.getContext().get_viewable()
        if version is None:
            return {}
        glossaryEntries = {}
        docElement = version.content.firstChild
        nodes = docElement.getElementsByTagName('link')
        for node in nodes:
            path = node.getAttribute('url')
            url, obj = glossaryLinkHelper(version, path)
            if obj != None and \
               obj.meta_type == 'Silva Glossary Entry':
                uid = '/'.join(obj.getPhysicalPath())
                glossaryEntries[uid] = obj
        return glossaryEntries


class GhostGlossarizerAdapter(GlossarizerAdapter):
    """Adapter to search for ghosted objects (IGhost)"""

    def getGlossaryEntries(self):
        """Get the glossary entries as a dictionary for Ghosts

        Can be used for ghost objects that implement IGhost. The method
        will look for the target object and call the adapter specific
        to that object.
        The key of the dictionary is the absolute path of the associated
        glossary entry. The value is the Glossary Entry object itself.
        """
        ghost_version = self.getContext().get_viewable()
        if ghost_version is None:
            return {}
        target = ghost_version.get_haunted()
        if target is None:
            return {}
        glossarizable = getGlossarizerAdapter(target)
        return glossarizable.getGlossaryEntries()


class ContainerGlossarizerAdapter(GlossarizerAdapter):
    """Adapter to get the indexes for containers"""
    def getGlossaryEntries(self):
        index = self.getContext().index
        return getGlossarizerAdapter(index).getGlossaryEntries()
    
def getGlossarizerAdapter(context):
    """Gets the glossarizer adapter for an objects"""
    if context.meta_type == 'Silva Document':
        return DocumentGlossarizerAdapter(context).__of__(context)
    if context.meta_type == 'dLCMS Content Element':
        return DocumentGlossarizerAdapter(context).__of__(context)
    if IGhost.isImplementedBy(context):
        return GhostGlossarizerAdapter(context).__of__(context)
    if IContainer.isImplementedBy(context):
        return ContainerGlossarizerAdapter(context).__of__(context) 
    return GlossarizerAdapter(context).__of__(context)

