# $Id: helpers.py 286 2005-12-12 16:51:00Z schluep $
# Copyright (c) 2005, Samuel Schluep, ETH Zurich. All rights reserved.
# See also LICENSE.txt

# Python
import string
import re
# Zope
from AccessControl import ModuleSecurityInfo
# Silva
from Products.Silva import mangle
# Silva Document
from Products.SilvaDocument import mixedcontentsupport

module_security = ModuleSecurityInfo('Products.SilvaGlossary.helpers')

__allow_access_to_unprotected_subobjects__ = 1


module_security.declarePublic('Id')
class Id(mangle.Id):
    """Utility class to Modifiy and validate ids

    Extends the Silva mangle.Id class by a cook method that guarantees to
    produces a valid id (well I hope so).
    """

    def cook_valid(self):
        """Makes a valid id.

        Silva's mangle cook method does no check for RESERVED_PREFIXes,
        RESERVED keywords, IN_USE_CONTENT or IN_USE_ASSET, or
        RESERVED_POSTFIXes. Strips all leading and trailing '_', alters
        reserved prefixes by the letter 'A', and adds an number for reserved
        keywords or ids in use.

        returns self
        """
        self.cook()
        # Strip leading '_'
        self._maybe_id = self._maybe_id.lstrip('_')
        self._swapcase_flag = None
        self._cook_helper(0, 0)


    def _cook_helper(self, counter, recursion_counter):
        """Recursively modifies the id based on the validation result"""
        validation_result = self._validate()
        recursion_counter += 1
        if validation_result == mangle.Id.OK:
            return
        elif recursion_counter > 9:
            # Catch an infinite recursion sequence
            raise StandardError('No valid id could be found')
        elif validation_result == mangle.Id.CONTAINS_BAD_CHARS:
            # Characters outside latin1 might not be converted, return as is
            # Error handling after extra validation
            return
        elif validation_result == mangle.Id.RESERVED_PREFIX:
            if not self._swapcase_flag:
                # Change the case of the first character
                self._maybe_id = self._maybe_id[0].swapcase() \
                               + self._maybe_id[1:]
                self._swapcase_flag = mangle.Id.RESERVED_PREFIX
            else:
                # If that did not help
                # Use original case
                if counter == 0:
                    self._maybe_id = self._maybe_id[0].swapcase() \
                                   + self._maybe_id[1:]
                # and append a number to the reserved word or prefix
                counter += 1
                parts = self._maybe_id.split('_')
                parts[0] = parts[0].rstrip('0123456789')
                parts[0] = parts[0] + str(counter)
                self._maybe_id = '_'.join(parts)
            self._cook_helper(counter, recursion_counter)
            return
        elif validation_result == mangle.Id.RESERVED:
            if self._swapcase_flag != mangle.Id.RESERVED:
                # Change the case of the first character
                self._maybe_id = self._maybe_id[0].swapcase() + \
                                 self._maybe_id[1:]
                self._swapcase_flag = mangle.Id.RESERVED
                counter = 0
            else:
                # If that did not help
                # Use original case
                if counter == 0:
                    self._maybe_id = self._maybe_id[0].swapcase() + \
                                     self._maybe_id[1:]
                # and append a number to the reserved word or prefix
                counter += 1
                self._maybe_id = self._maybe_id.rstrip('0123456789')
                self._maybe_id = self._maybe_id + str(counter)
            self._cook_helper(counter, recursion_counter)
            return
        elif validation_result == mangle.Id.IN_USE_CONTENT or \
             validation_result == mangle.Id.IN_USE_ASSET:
            # Id is already in use, append a number
            if self._swapcase_flag:
                # If case of first character has changed reset
                if counter == 0:
                    self._maybe_id = self._maybe_id[0].swapcase() \
                                   +  self._maybe_id[1:]
                if self._swapcase_flag == mangle.Id.RESERVED_PREFIX:
                    # Reset counter if it has been used after a prefix
                    counter = 0
                self._swapcase_flag = None
            # Append a new number
            counter += 1
            self._maybe_id = self._maybe_id.rstrip('0123456789')
            self._maybe_id = self._maybe_id + str(counter)
            self._cook_helper(counter, recursion_counter)
            return
        elif validation_result == mangle.RESERVED_POSTFIX:
            # The one possible reserved postfix is '__'. Remove the last '_'.
            self._maybe_id = self._maybe_id[:-1]
            self._cook_helper(counter, recursion_counter)
            return
        else:
            raise ValueError('Unknown id validation error code')


_url_match = re.compile(mixedcontentsupport.URL_PATTERN)

def glossaryLinkHelper(context, path):
    """Get the absolute path and the Silva or Zope object.
    
    If the path's target is a Silva or Zope object, the absolute path
    and the object is returned, otherwise the path is left as is and for
    the object 'None' is returned.
    
    Returns a tuple with the processed path and the Silva/Zope object.
    """
    # If path is empty (can it be?), just return it
    if path == '':
        return path, None
    # If it is a url already, return it:
    if _url_match.match(path):
        return path, None
    # Is it a query of anchor fragment? If so, return it
    if path[0] in ['?', '#']:
        return path, None
    # It is not an URL, query or anchor, so treat it as a path.
    # Try to traverse it to a Zope/Silva object and get the URL for that.
    splitpath = [p.encode('ascii','ignore') for p in path.split('/')]
    obj = context.restrictedTraverse(splitpath, None)
    if obj is None:
        # Was not found, maybe the link is broken, but maybe it's just
        # due to virtual hosting situations or whatever.
        return path, None
    if hasattr(obj.aq_base, 'absolute_url'):
        # There are some cases where the object we find
        # does not have the absolute_url method.
        return obj.absolute_url(), obj
    # In all other cases:
    return path, obj



